<?php

namespace Modules\Qc\Http\Controllers;

use Exception;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Qc\Entities\SampleInformation;
use Modules\Qc\Entities\SampleInformationAttachment;

class SampleInformationController extends Controller
{
    public function uploadAttachment(Request $request)
    {
        try {
            $token = $request->get('attachment_token');
            $file = $request->file('file');
            $file_name = $file->getClientOriginalName();
            $filename = uniqid() . '_' . time() . '.' . $file_name;
            $file->move(public_path('sample_information'), $filename);

            $sample_information_attachment = new SampleInformationAttachment();
            $sample_information_attachment->sample_information_id = 0;
            $sample_information_attachment->path = $filename;
            $sample_information_attachment->token = $token;
            $sample_information_attachment->save();

            $responseBody = $this->responseBody(true, "SampleInformationController", "uploaded", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "SampleInformationController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function allRecords()
    {
        try {
            $query = "SELECT
            qc_sample_information.*,
            qc_sample_information_attachments.path,
            hr_employee_sites.site
        FROM
            qc_sample_information_attachments
            INNER JOIN
            qc_sample_information
            ON
                qc_sample_information_attachments.sample_information_id = qc_sample_information.sample_information_id
            INNER JOIN
            hr_employee_sites
            ON
                qc_sample_information.factory = hr_employee_sites.site_id
                GROUP BY
            qc_sample_information.sample_information_id";

            $records = DB::select($query);
            $result = array();
            foreach ($records as $record) {
                array_push($result, [
                    "sample_information_id" => $record->sample_information_id,
                    "reference_no" => $record->reference_no,
                    "date" => $record->date,
                    "factory" => $record->site,
                    "description" => $record->description,
                    "path" => $record->path,
                ]);
            }
            $responseBody = $this->responseBody(true, "SampleInformationController", "allRecords", $result);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "SampleInformationController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function saveRecord(Request $request)
    {
        try {
            $sample = new SampleInformation();
            $sample->user_id = Auth::user()->id;
            $sample->reference_no = $request->get('reference_no');
            $sample->date = $request->get('date');
            $sample->factory = $request->get('factory');
            $sample->description = $request->get('description');
            if ($sample->save()) {
                $token = $request->get('attachment_token');
                $attachment = SampleInformationAttachment::where('token', '=', $token)->first();
                if ($attachment) {
                    $attachment->sample_information_id = $sample->sample_information_id;
                    $attachment->update();
                }
            }
            $responseBody = $this->responseBody(true, "SampleInformationController", "saveRecord", $attachment);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "SampleInformationController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function update(Request $request, $id)
    {
        try {
            $sample = SampleInformation::find($id);
            $sample->user_id = Auth::user()->id;
            $sample->reference_no = $request->get('reference_no');
            $sample->date = $request->get('date');
            $sample->factory = $request->get('factory');
            $sample->description = $request->get('description');
            if ($sample->update()) {
                $token = $request->get('attachment_token');
                $attachments = SampleInformationAttachment::where('token', '=', $token)->get();
                if (count($attachments) > 1) {
                    if ($this->delete($id)) {
                        $attachment = SampleInformationAttachment::where('token', '=', $token)->first();
                        $attachment->sample_information_id = (int)$id;
                        $attachment->update();
                    }
                }
                $responseBody = $this->responseBody(true, "SampleInformationController", "updated", count($attachments));
            }
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "SampleInformationController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }




    public function delete($id)
    {
        try {
            $attachment = SampleInformationAttachment::where('sample_information_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            return true;
        } catch (Exception $exception) {
            return false;
        }
    }


    public function deleteRecord($id)
    {
        try {
            $record = SampleInformation::where('sample_information_id', '=', $id)->get();
            foreach ($record as $data) {
                $data->delete();
            }
            $attachment = SampleInformationAttachment::where('sample_information_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            $responseBody = $this->responseBody(true, "SampleInformationController", "deleted", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "SampleInformationController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function getData($id)
    {
        try {
            $query = "SELECT
            qc_sample_information.*,
            qc_sample_information_attachments.token
        FROM
            qc_sample_information_attachments
            INNER JOIN
            qc_sample_information
            ON
                qc_sample_information_attachments.sample_information_id = qc_sample_information.sample_information_id
        WHERE
            qc_sample_information.sample_information_id = {$id}";

            $sample =  DB::select($query);

            $responseBody = $this->responseBody(true, "SampleInformationController", "getData", $sample[0]);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "SampleInformationController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }



    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
