<?php

namespace Modules\Qc\Http\Controllers;

use Exception;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Qc\Entities\WeedContaminationRecord;
use Modules\Qc\Entities\WeedContaminationRecordAttachment;

class WeedContaminationRecordController extends Controller
{
    public function uploadAttachment(Request $request)
    {
        try {
            $token = $request->get('attachment_token');
            $file = $request->file('file');
            $file_name = $file->getClientOriginalName();
            $filename = uniqid() . '_' . time() . '.' . $file_name;
            $file->move(public_path('weed_contamination'), $filename);

            $weed_contamination_attachment = new WeedContaminationRecordAttachment();
            $weed_contamination_attachment->weed_contamination_id = 0;
            $weed_contamination_attachment->path = $filename;
            $weed_contamination_attachment->token = $token;
            $weed_contamination_attachment->save();

            $responseBody = $this->responseBody(true, "WeedContaminationRecordController", "uploaded", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "WeedContaminationRecordController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function allRecords()
    {
        try {
            $query = "SELECT
            qc_weed_contamination_records.*,
            qc_weed_contamination_record_attachments.path,
            hr_employee_sites.site
        FROM
            qc_weed_contamination_record_attachments
            INNER JOIN
            qc_weed_contamination_records
            ON
                qc_weed_contamination_record_attachments.weed_contamination_id = qc_weed_contamination_records.weed_contamination_id
            INNER JOIN
            hr_employee_sites
            ON
                qc_weed_contamination_records.factory = hr_employee_sites.site_id
                GROUP BY
            qc_weed_contamination_records.weed_contamination_id";

            $records = DB::select($query);
            $result = array();
            foreach ($records as $record) {
                array_push($result, [
                    "weed_contamination_id" => $record->weed_contamination_id,
                    "reference_no" => $record->reference_no,
                    "date" => $record->date,
                    "factory" => $record->site,
                    "description" => $record->description,
                    "path" => $record->path,
                ]);
            }
            $responseBody = $this->responseBody(true, "WeedContaminationRecordController", "allRecords", $result);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "WeedContaminationRecordController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function saveRecord(Request $request)
    {
        try {
            $weed = new WeedContaminationRecord();
            $weed->user_id = Auth::user()->id;
            $weed->reference_no = $request->get('reference_no');
            $weed->date = $request->get('date');
            $weed->factory = $request->get('factory');
            $weed->description = $request->get('description');
            if ($weed->save()) {
                $token = $request->get('attachment_token');
                $attachment = WeedContaminationRecordAttachment::where('token', '=', $token)->first();
                if ($attachment) {
                    $attachment->weed_contamination_id = $weed->weed_contamination_id;
                    $attachment->update();
                }
            }
            $responseBody = $this->responseBody(true, "WeedContaminationRecordController", "saveRecord", $attachment);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "WeedContaminationRecordController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function update(Request $request, $id)
    {
        try {
            $weed = WeedContaminationRecord::find($id);
            $weed->user_id = Auth::user()->id;
            $weed->reference_no = $request->get('reference_no');
            $weed->date = $request->get('date');
            $weed->factory = $request->get('factory');
            $weed->description = $request->get('description');
            if ($weed->update()) {
                $token = $request->get('attachment_token');
                $attachments = WeedContaminationRecordAttachment::where('token', '=', $token)->get();
                if (count($attachments) > 1) {
                    if ($this->delete($id)) {
                        $attachment = WeedContaminationRecordAttachment::where('token', '=', $token)->first();
                        $attachment->weed_contamination_id = (int)$id;
                        $attachment->update();
                    }
                }
                $responseBody = $this->responseBody(true, "WeedContaminationRecordController", "updated", count($attachments));
            }
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "WeedContaminationRecordController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }




    public function delete($id)
    {
        try {
            $attachment = WeedContaminationRecordAttachment::where('weed_contamination_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            return true;
        } catch (Exception $exception) {
            return false;
        }
    }


    public function deleteRecord($id)
    {
        try {
            $record = WeedContaminationRecord::where('weed_contamination_id', '=', $id)->get();
            foreach ($record as $data) {
                $data->delete();
            }
            $attachment = WeedContaminationRecordAttachment::where('weed_contamination_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            $responseBody = $this->responseBody(true, "WeedContaminationRecordController", "deleted", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "WeedContaminationRecordController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function getData($id)
    {
        try {
            $query = "SELECT
            qc_weed_contamination_records.*,
            qc_weed_contamination_record_attachments.token
        FROM
            qc_weed_contamination_record_attachments
            INNER JOIN
            qc_weed_contamination_records
            ON
                qc_weed_contamination_record_attachments.weed_contamination_id = qc_weed_contamination_records.weed_contamination_id
        WHERE
            qc_weed_contamination_records.weed_contamination_id = {$id}";

            $weed =  DB::select($query);

            $responseBody = $this->responseBody(true, "WeedContaminationRecordController", "getData", $weed[0]);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "WeedContaminationRecordController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }



    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
